#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/spi/spidev.h>

void faultExit(const char *s)
{
    printf("%s\n", s);
    exit(1);
}

int main()
{
    unsigned char mode      = SPI_MODE_3;   //
    unsigned char bits      = 8;            // 8 bits per word
    unsigned int  maxSpeed  = 10000;        //

    char spiDevice[]        = "/dev/spidev0.0";

    int fd  = 0;
    int ret = 0;

    fd = open(spiDevice, O_RDWR);
    if (fd <= 0)
        faultExit("unable to open device");

    ret = ioctl(fd, SPI_IOC_WR_MODE, &mode);
    if (ret < 0)
        faultExit("unable to set spi mode");

    ret = ioctl(fd, SPI_IOC_RD_MODE, &mode);
    if (ret < 0)
        faultExit("unable to get spi mode");

    ret = ioctl(fd, SPI_IOC_WR_BITS_PER_WORD, &bits);
    if (ret < 0)
        faultExit("unable to set bits count");
    
    ret = ioctl(fd, SPI_IOC_RD_BITS_PER_WORD, &bits);
    if (ret < 0)
        faultExit("unable to get bits count");
    
    ret = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &maxSpeed);
    if (ret < 0)
        faultExit("unable to set max speed");

    ret = ioctl(fd, SPI_IOC_RD_MAX_SPEED_HZ, &maxSpeed);
    if (ret < 0)
        faultExit("unable to get max speed");

    printf("spi mode:  %d\n", mode);
    printf("bits:      %d\n", bits);
    printf("max speed: %d Hz\n", maxSpeed);

    unsigned char tx[] = {0xFF, 0x7F, 0x3F, 0x1F};
    unsigned char rx[4];

    struct spi_ioc_transfer tr = {
        .tx_buf         = (unsigned long)tx,
        .rx_buf         = (unsigned long)rx,
        .len            = 4,
        .delay_usecs    = 0,
        .speed_hz       = 0,
        .bits_per_word  = bits
    };

    while (1) {
        ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
        printf("TX: ");
        for (int i = 0; i < 4; i++) {
            printf("%#02X ", tx[i]);
        }
        printf("\n");
        printf("RX: ");
        for (int i = 0; i < 4; i++) {
            printf("%#02X ", rx[i]);
        }
        printf("\n");
        printf("\n");
    }

    close(fd);

    return 0;
}